/*---------------------------------------------------------------*
 *
 * NOTE: This was written for an earlier version of Pasokon TV
 * which could only handle 320 x 240 images of 32k colors.
 * 
 * Version 3 uses 640 x 480 images with 16 million colors and
 * can read a wider variety of file types.  Keep this in mind
 * if you use this as a starting point for writing your own
 * application.
 *
 *---------------------------------------------------------------*/

/*
 * Name:        FLAG.C
 *
 * Date:        February 1993.
 *
 * Purpose:     Sample program for use with Pasokon TV.
 *
 * Description: This will generate a flag for the specified country.
 *
 * Usage:       FLAG country out-file
 *              where,
 *                      country is a country whose flag is VERY
 *                                      easy to generate.
 *                      out-file is name of file to generate.
 *
 *              Read the PKTV.CFG documentation for an explanation 
 *              of how to run it from a user-defined menu item.
 */

#include <stdio.h>

/*
 * A pixel is 16 bits with the following pattern:
 *      0 r r r r r g g g g g b b b b b
 *
 * Each of the R, G, or B components can vary from 0 to 31.
 * Here are some macros to pack 3 components into an integer
 * and to extract them.
 */
#define R_MAX 31
#define G_MAX 31
#define B_MAX 31

#define rgb2color(r,g,b) (((r)<<10)|((g)<<5)|(b))

#define r_part(c) (((c)>>10)&R_MAX)
#define g_part(c) (((c)>>5)&G_MAX)
#define b_part(c) ((c)&B_MAX)

/*
 * Pasokon TV always uses images of 320 columns (pixels/line)
 * and 240 rows or lines.
 */
#define NCOLS 320
#define NROWS 240

/*
 * Values for some popular colors.
 */
#define BLACK   rgb2color (0, 0, 0)
#define RED     rgb2color (R_MAX, 0, 0)
#define ORANGE  rgb2color (R_MAX, G_MAX/2, 0)
#define YELLOW  rgb2color (R_MAX, G_MAX, 0)
#define GREEN   rgb2color (0, G_MAX, 0)
#define BLUE    rgb2color (0, 0, B_MAX)
#define MAGENTA rgb2color (R_MAX, 0, B_MAX)
#define CYAN    rgb2color (0, G_MAX, B_MAX)
#define GRAY    rgb2color (R_MAX/2, G_MAX/2, B_MAX/2)
#define WHITE   rgb2color (R_MAX, G_MAX, B_MAX)

/*
 * And the colors of very easy to generate flags.
 */
static struct {
	char *name;
	int c1;
	int c2;
	int c3;
} countries[] = {
	{ "Chad",               BLUE, YELLOW, RED },
	{ "Ireland",            GREEN, WHITE, ORANGE },
	{ "Italy",              GREEN, WHITE, RED },
	{ "Nigeria",            GREEN, WHITE, GREEN },
	{ "Belgium",            BLACK, YELLOW, RED },
	{ "France",             BLUE, WHITE, RED },
	{ "Libya",              GREEN, GREEN, GREEN },
	{ "Romania",            BLUE, YELLOW, RED },
	{ "Mali",               GREEN, YELLOW, RED },
	{ "Guinea",             RED, YELLOW, GREEN },
	{ NULL,                 0, 0, 0 } };


main (argc, argv)
int argc;
char *argv[];
{
	int i, found, row, col;
	FILE *fp;


	if (argc < 3) {
	    fprintf (stderr, "Not enough command line arguments.\n");
	    fprintf (stderr, "Press 'Enter' to continue ...\n");
	    getchar ();
	    exit (0);
	}

/*
 * Look up country name in table.
 */
	found = -1;
	for (i=0; countries[i].name != NULL; i++) {
	    if (stricmp(argv[1], countries[i].name) == 0) {
		found = i;
		break;
	    }
	}

	if (found < 0) {
	    fprintf (stderr, "Don't know how to make flag for %s.\n", argv[1]);
	    fprintf (stderr, "Press 'Enter' to continue ...\n");
	    getchar ();
	    exit (0);
	}

/*
 * Create a Targa format image file.
 */
	fp = fopen (argv[2], "wb");
	if (fp == NULL) {
	    fprintf (stderr, "Can't open %s for write.\n", argv[2]);
	    fprintf (stderr, "Press 'Enter' to continue ...\n");
	    getchar ();
	    exit (0);
	}

/*
 * The file header.
 */
	fputc (0, fp);                  /* Id length. */
	fputc (0, fp);                  /* No color map. */
	fputc (2, fp);                  /* Image type 2 = RGB. */
	fputc (0, fp);                  /* Color map stuff. */
	fputc (0, fp);
	fputc (0, fp);
	fputc (0, fp);
	fputc (0, fp);
	fputc (0, fp);                  /* Image origin. */
	fputc (0, fp);
	fputc (0, fp);
	fputc (0, fp);
	fputc (NCOLS & 0xFF, fp);       /* Dimensions. */
	fputc (NCOLS >> 8, fp);
	fputc (NROWS & 0xFF, fp);
	fputc (NROWS >> 8, fp);
	fputc (16, fp);                 /* Bits per pixel. */
	fputc (1 << 5, fp);             /* Scan top to bottom. */

/*
 * The image.
 */
	for (row=0; row<NROWS; row++) {
	    for (col=0; col<NCOLS/3; col++) {
		fputc (countries[found].c1 & 0xFF, fp);
		fputc (countries[found].c1 >> 8, fp);
	    }
	    for (col=NCOLS/3; col<2*NCOLS/3; col++) {
		fputc (countries[found].c2 & 0xFF, fp);
		fputc (countries[found].c2 >> 8, fp);
	    }
	    for (col=2*NCOLS/3; col<NCOLS; col++) {
		fputc (countries[found].c3 & 0xFF, fp);
		fputc (countries[found].c3 >> 8, fp);
	    }
	}
	fclose (fp);
}
